/*
  lpt.c
  Matlib 2010-11-11
*/


#ifndef _LPT_C
#define _LPT_C 1

#include <linux/parport.h>
#include "tpi.h"
#include "lpt.h"


static int (*lpt_in) (unsigned char *);
static int (*lpt_out) (unsigned char);
static int (*lpt_sin) (unsigned char *);
static int (*lpt_cout) (unsigned char);
unsigned char lpt_data, lpt_control;

static int lpt_init (int (*in) (unsigned char *), int (*out) (unsigned char), 
   int (*sin) (unsigned char *),
   int (*cin) (unsigned char *), int (*cout) (unsigned char))
{
  lpt_in = in;
  lpt_out = out;
  lpt_sin = sin;
  lpt_cout = cout;
  if ((lpt_in (&lpt_data) != 0) || (cin (&lpt_control) != 0))
    return -1;
  lpt_control &= ~(LPT_VPP | LPT_RESET);
  if (lpt_cout (lpt_control) != 0)
    return -1;
  hw_setport = lpt_setport;
  hw_readport = lpt_readport;
  hw_power = lpt_power;
  hw_reset = lpt_reset;
  hw_ports = 4;
  return 0;
}



static int lpt_setport (unsigned pin, unsigned state)
{
  if (pin >= 4)
    return -1;
  lpt_data &= ~(3 << (2 * pin));
  lpt_data |= (state & 3) << (2 * pin);
  return lpt_out (lpt_data);
}

static int lpt_readport (unsigned pin, int *value)
{
  unsigned char status;
  static const unsigned char PINS [4] = { LPT_P0IN, LPT_P1IN, LPT_P2IN, 
     LPT_P3IN };
     
  if (pin >= 4)
    return -1;
  if (lpt_sin (&status) != 0)
    return -1;
  *value = ((status ^ LPT_INXOR) & PINS [pin]) != 0 ? 1 : 0;
  return 0;
}


static int lpt_power (int state)
{
  if (state)
    lpt_control |= LPT_VPP;
  else
    lpt_control &= ~LPT_VPP;
  return lpt_cout (lpt_control);
}


static int lpt_reset (int state)
{
  if (state)
    lpt_control |= LPT_RESET;
  else
    lpt_control &= ~LPT_RESET;
  return lpt_cout (lpt_control);
}


#endif

