/*
  tpi.c
  Matlib 2010-10-15
*/


#include "tpi.h"



static int tpi_enter_lowv ()
{
  // turn on
  if ((hw_reset (0) != 0) || (hw_power (1) != 0) 
     || (hw_setport (TPIRESET, HW_ON) != 0))
    return -1;
  // wait t_TOUT = 128 ms
  msleep (150);
  // set !RESET to 0
  if ((hw_setport (TPIRESET, HW_OFF) != 0) ||
     (hw_setport (TPIDATA, HW_ON) != 0))
    return -1;
  return 0;
}

static int tpi_enter_hiv ()
{
  // turn on
  if ((hw_reset (0) != 0) || (hw_power (1) != 0) 
     || (hw_setport (TPIRESET, HW_ON) != 0))
    return -1;
  msleep (10);
  // set !RESET to 12 V
  if ((hw_setport (TPIRESET, HW_HIZ) != 0) || (hw_reset (1) != 0) ||
     (hw_setport (TPIDATA, HW_ON) != 0))
    return -1;
  return 0;
}


ARGUMENT_CALLBACK (tpi_hiv)
{
  arguments [ARG_N_TPIE].value = tpi_enter_hiv;
}


static int tpi_init ()
{
  unsigned i;
  printf ("Entering TPI mode... ");
  for (i = 0; i < hw_ports; i ++)
    if ((i != TPIRESET) && (hw_setport (i, HW_HIZ) != 0))
      return -1;
  ARG_TPIE ();
  // wait t_RST = 2 µs
  msleep (10);
  // wait 16 CLKs
  for (i = 0; i < 16; i ++)   
    if (tpi_clock () != 0)
      return -1;
  if ((tpi_checksig () != 0) || (tpi_setgt (2) != 0) || (tpi_skey () != 0))
    return -1;
  printf ("done\n");
  return 0;
}



static int tpi_clock ()
{
  if (hw_setport (TPICLK, HW_OFF) != 0)
    return -1;
  usleep_cal (ARG_CLKT);
  if (hw_setport (TPICLK, HW_ON) != 0)
    return -1;
  usleep_cal (ARG_CLKT);
  stat_bits ++;
  return 0;
}

static int tpi_senddata (int bit)
{
  if ((hw_setport (TPIDATA, bit == 0 ? HW_OFF : HW_ON) != 0)
     || (tpi_clock () != 0))
    return -1;
  return 0;
}

static int tpi_readdata (int *bit)
{
  if ((tpi_clock () != 0) || (hw_readport (TPIDATA, bit) != 0))
    return -1;
  return 0;
}
     
static int tpi_sendbyte (unsigned char b)
{
  unsigned i;
  unsigned char cs;
  
  stat_in ++;
  for (i = 0; i < 4; i ++)
    if (tpi_senddata (1) != 0)
      return -1;
  // start bit
  if (tpi_senddata (0) != 0)
    return -1; 
  cs = 0;
  // 8 data bits
  for (i = 0; i < 8; i ++)
  {
    if (tpi_senddata (b & 1) != 0)
      return -1;
    cs ^= b & 1;
    b >>= 1;
  }
  // parity + 2 stop bits
  if ((tpi_senddata (cs & 1) != 0) || (tpi_senddata (1) != 0)
     || (tpi_senddata (1) != 0))
    return -1;
  return 0;
}

static int tpi_recvbyte (unsigned char *b)
{
  unsigned i, p;
  int bit, s1, s2;
  
  stat_out ++;
  hw_setport (TPIDATA, HW_HIZ);
  i = 0;
  *b = 0;
  do
  {
    if (++ i > 150)
    {
      printf ("receive timeout\n");
      return -1;
    }
    if (tpi_readdata (&bit) != 0)
      return -1;
  }
  while (bit != 0);
  p = 0;
  for (i = 0; i < 8; i ++)
  {
    if (tpi_readdata (&bit) != 0)
      return -1;
    *b >>= 1;
    *b |= (bit != 0) ? 0x80 : 0;
    p ^= (*b) & 0x80;
  }
  if (tpi_readdata (&bit) != 0)
    return -1;
  p ^= (bit != 0) ? 0x80 : 0;
  if ((tpi_readdata (&s1) != 0) || (tpi_readdata (&s2) != 0))
    return -1;
  for (i = 0; i < 4; i ++)
    if (tpi_readdata (&bit) != 0)
      return -1;
  if (p != 0)
  {
    printf ("receive: parity error\n");
    return -1;
  }
  if (!s1)
  {
    printf ("receive: 1st stop bit missing\n");
    return -1;
  }
  if (!s2)
  {
    printf ("receive: 2nd stop bit missing\n");
    return -1;
  }
  return 0;
}




/* static int tpi_break ()
{
  unsigned i;
  printf ("Sending break... ");
  for (i = 0; i < 4; i ++)
    if (tpi_senddata (1) != 0)
      return -1;
  for (i = 0; i < 4; i ++)
    if (tpi_senddata (0) != 0)
      return -1;
  for (i = 0; i < 4; i ++)
    if (tpi_senddata (1) != 0)
      return -1;
  for (i = 0; i < 4; i ++)
    if (tpi_senddata (0) != 0)
      return -1;
  for (i = 0; i < 4; i ++)
    if (tpi_senddata (1) != 0)
      return -1;
  printf ("done\n");
  return 0;
}*/



// check signature
static int tpi_checksig ()
{
  unsigned char b;
  printf ("signature ");
  if ((tpi_sendbyte (0x8f) == 0) && (tpi_recvbyte (&b) == 0))
  {
    printf ("\010=0x%02x ", b);
    if (b == 0x80)
      return 0;
    else
      printf ("\nWrong signature.\n");
  }
  return -1;
}



// wait for status register NVMEN
static int tpi_waitnvmen ()
{
  unsigned i;
  unsigned char b;
  
  i = 0;
  printf ("status register ");
loop:
  if ((tpi_sendbyte (0x80) == 0) && (tpi_recvbyte (&b) == 0))
  {
    if (b == 0)
    {
      if (i ++ < 10)
      {
        printf ("%d ", i);
        goto loop;
      }
      printf ("\nTimeout exceeded while enabling write mode\n");
      return -1;
    }
    else if (b == 0x02)
      return 0;
    else
      printf ("\nInvalid value received 0x%02X.\n", b);
  }
  return -1;
}


// set status register NVMEN
/*static int tpi_setnvmen (int sr)
{
  if (sr == 0)
  {
    printf ("Disabling NVR write... ");
    if ((tpi_sendbyte (0xc0) != 0) || (tpi_sendbyte (0) != 0))
      return -1;
  }
  else
  {
    printf ("Enabling NVR write... ");
    if ((tpi_sendbyte (0xc0) != 0) || (tpi_sendbyte (0x02) != 0))
      return -1;
  }
  printf ("done\n");
  return 0;
}*/



// set guard time
static int tpi_setgt (unsigned gt)
{
  unsigned div;
  printf ("guard time ");
  div = 8;
  do
  {
    div --;
    if ((gt == 0) || (128 / (1 << div) == gt))
    {
      if ((tpi_sendbyte (0xc2) != 0) || (tpi_sendbyte (div) != 0))
        return -1;
      printf ("\010=%uCLKs ", gt);
      return 0;
    }
  }
  while (div > 0);
  printf ("\nInvalid guard time value %u\n", gt);
  return -1;
}





// send key
static int tpi_skey ()
{
  printf ("key ");
  if ((tpi_sendbyte (0xe0) != 0) ||
     (tpi_sendbyte (0xff) != 0) ||
     (tpi_sendbyte (0x88) != 0) ||
     (tpi_sendbyte (0xd8) != 0) ||
     (tpi_sendbyte (0xcd) != 0) ||
     (tpi_sendbyte (0x45) != 0) ||
     (tpi_sendbyte (0xab) != 0) ||
     (tpi_sendbyte (0x89) != 0) ||
     (tpi_sendbyte (0x12) != 0))
    return -1;
  return tpi_waitnvmen ();
}





// serial load
static int tpi_sld (unsigned char *b)
{
  if ((tpi_sendbyte (0x20) != 0) || (tpi_recvbyte (b) != 0))
    return -1;
  return 0;
}

// serial load PR++
static int tpi_sldp (unsigned char *b)
{
  if ((tpi_sendbyte (0x24) != 0) || (tpi_recvbyte (b) != 0))
    return -1;
  return 0;
}



// serial store
static int tpi_sst (unsigned char b)
{
  if ((tpi_sendbyte (0x60) != 0) || (tpi_sendbyte (b) != 0))
    return -1;
  return 0;
}

// serial store PR++
static int tpi_sstp (unsigned char b)
{
  if ((tpi_sendbyte (0x64) != 0) || (tpi_sendbyte (b) != 0))
    return -1;
  return 0;
}



// serial store to pointer register
static int tpi_sstpr (uint16_t pr)
{
  if ((tpi_sendbyte (0x68) != 0) || (tpi_sendbyte (pr) != 0)
     || (tpi_sendbyte (0x69) != 0) || (tpi_sendbyte (pr >> 8) != 0))
    return -1;
  return 0;
}




// serial in from I/O space
static int tpi_sin (uint8_t address, unsigned char *b)
{
  unsigned char cmd;
  
  *b = 0;
  if ((address & 0xc0) != 0)
  {
    printf ("SIN: address 0x%02X out of range\n", address);
    return -1;
  }
  cmd = address << 1;
  cmd &= 0xf0;
  cmd |= address & 0x0f;
  cmd |= 0x10;
  if ((tpi_sendbyte (cmd) != 0) || (tpi_recvbyte (b) != 0))
    return -1;
  return 0;
}


// serial out from I/O space
static int tpi_sout (uint8_t address, unsigned char b)
{
  unsigned char cmd;
  if ((address & 0xc0) != 0)
  {
    printf ("SOUT: address 0x%02X out of range\n", address);
    return -1;
  }
  cmd = address << 1;
  cmd &= 0xf0;
  cmd |= address & 0x0f;
  cmd |= 0x90;
  if ((tpi_sendbyte (cmd) != 0) || (tpi_sendbyte (b) != 0))
    return -1;
  return 0;
}

